require "cross_spec/version"
require "rubygems"
require "bundler/setup"
Bundler.setup
require "distributed_tracing"
require "json"
require "kafka"
require "securerandom"
require "timeout"

module CrossSpec
  class FailureBeacon < StandardError; end
  FAIL_TASK = "cross_spec:failure".freeze

  autoload :Beacon,           "cross_spec/beacon"
  autoload :Client,           "cross_spec/client"
  autoload :Config,           "cross_spec/config"
  autoload :KafkaClient,      "cross_spec/kafka_client"
  autoload :Message,          "cross_spec/message"
  autoload :Spec,             "cross_spec/spec"
  autoload :SpecWaiting,      "cross_spec/spec"
  autoload :ServiceMessage,   "cross_spec/service_message"
  autoload :TaskMessage,      "cross_spec/task_message"

  def self.configure
    @config = Config.new
    yield(@config)
  end

  def self.config
    return @config if defined?(@config)
    @config = Config.new
    @config
  end

  def self.handle(service, &block)
    config.handlers[service] = Proc.new do |*args|
      block.call(*args)
      nil
    end
  end

  def self.listen!
    config.client.listen!
  end

  def self.failure_beacon!(data = nil)
    beacon(FAIL_TASK, data)
  end

  def self.beacon(task, data = nil, biomarkers = nil)
    str = JSON.dump(
      task: task,
      biomarkers: biomarkers || DistributedTracing.get,
      data: data
    )
    config.client.broadcast(str)
  end
end
